% This experiment testing the effect of inner-distance shape context
%	on swedish leaf database containing 15x75 images
% Haibin Ling

clear;
addpath ../common_innerdist;


%------ Parameters ----------------------------------------------
ifig		= -1;
sdir		='data/';
n_class		= 15;
n_obj		= 75;

%-- shape context parameters
n_dist		= 8;
n_theta		= 12;
bTangent	= 1;
bSmoothCont	= 1;
n_contsamp	= 128;
match_thre	= .6;

%-- FILEs --------------------------------------------
sDisAngSamp	= ['_' i2s(n_dist,2) 'x' i2s(n_theta,2) 'x' i2s(n_contsamp,3)];
sCont		= [sdir 'swd_cont_' i2s(n_contsamp) '.mat'];
s_swd_sc	= [sdir 'IDSC' sDisAngSamp '_tan' i2s(bTangent) '.mat'];

sRecord		= [sdir 'IDSC_res' sDisAngSamp '.cpp'];
fRecord		= fopen(sRecord,'w');

fprintf('%s,\n\n', sDisAngSamp);
fprintf(fRecord,'%s,\n\n', sDisAngSamp);


%-- Extract contours -------------------------------------------------------------
if 1
	[swd_cont] = batch_contour_f(sdir);
	[swd_cont] = batch_contour_f(sdir,n_contsamp);
% 	keyboard;
end

%-- Compute or Load all the shape context data ------------------------------------
if 1
	[swd_sc] = batch_comp_swd_sc(sdir,sCont,n_dist, n_theta, bTangent, bSmoothCont);
	save(s_swd_sc, 'swd_sc');
end


%-- KNN classifying each object by comparing its SC to training objects -----------------
fprintf('KNN, Classifying each object by comparing its SC to training objects .............\n');
fprintf(fRecord,'KNN, Classifying each object by comparing its SC to training objects .............\n');
if ~exist('swd_cont')			load(sCont);			end
if ~exist('swd_sc')				load(s_swd_sc);		end

n_training	= 25;
n_testing	= 75-n_training;

n_top		= 5;		% store result of n_top matching for eaching testing object
match_res	= cell(n_class,2);
tmp_dists	= zeros(n_class*n_training,2);
idx_class	= repmat(1:n_class,1,n_obj);
idx_obj		= ceil((1:n_class*n_obj)/n_class);

n_hits		= zeros(n_class,2);
scores		= zeros(size(n_hits));
% w			= [1,0];
num_start	= 1;
search_step	= 1;

for iC=1:n_class
	fprintf('\nClass %2d ..................................', iC);
	fprintf(fRecord,'\nClass %2d ..................................', iC);

	match_res{iC}	= zeros(n_testing,n_top*3);		% each triple is (classID,objID,dist)
	for iO=n_training+1:n_obj
		tic;
		sc_test	= swd_sc{iC,iO};
		V1		= swd_cont{iC,iO};
		
		for iTrain=1:n_class*n_training
			sc_train	= swd_sc{iTrain};
			V2			= swd_cont{iTrain};
			[dis_sc,match_cost,n_match] = dist_bw_sc_swd( sc_test,sc_train, V1,V2, search_step, num_start, match_thre, ifig);
			tmp_dists(iTrain,:)	= [dis_sc,match_cost];
		end

		[topdis,topidx]		= sort(tmp_dists,1);
		topdis	= topdis(1:n_top,:);
		topidx	= topidx(1:n_top,:);
		topCs	= idx_class(topidx);
		topOs	= idx_obj(topidx);
		
		match_res{iC,1}(iO-n_training,:) = reshape([topCs(:,1),topOs(:,1),topdis(:,1)]', 1,n_top*3);
		match_res{iC,2}(iO-n_training,:) = reshape([topCs(:,2),topOs(:,2),topdis(:,2)]', 1,n_top*3);
		
		bestC	= topCs(1,:);
		fprintf('\n%2d: ', iO);
		fprintf('chamfer: ');		fprintf('%2d ', topCs(:,1));
		fprintf(', ');				fprintf('%2d ', topOs(:,1));
		fprintf('  DP: ');			fprintf('%2d ', topCs(:,2));
		fprintf(', ');				fprintf('%2d ', topOs(:,2));
		fprintf(', %2d sec',round(toc));

		fprintf(fRecord,'\n%2d: ', iO);
		fprintf(fRecord,'chamfer: ');		fprintf(fRecord,'%2d ', topCs(:,1));
		fprintf(fRecord,', ');				fprintf(fRecord,'%2d ', topOs(:,1));
		fprintf(fRecord,'  DP: ');			fprintf(fRecord,'%2d ', topCs(:,2));
		fprintf(fRecord,', ');				fprintf(fRecord,'%2d ', topOs(:,2));
		fprintf(fRecord,', %2d sec',round(toc));
	end

	n_hits(iC,1)	= sum( match_res{iC,1}(:,1)==iC );		% 1-NN
	n_hits(iC,2)	= sum( match_res{iC,2}(:,1)==iC );
	scores(iC,:)	= n_hits(iC,:)/n_testing;
	fprintf('\n  Class %2d:  chamfer: %2d / %.4f,    DP: %2d / %.4f\n', ...
				iC,n_hits(iC,1),scores(iC,1),n_hits(iC,2),scores(iC,2));
	fprintf(fRecord,'\n  Class %2d:  chamfer: %2d / %.4f,    DP: %2d / %.4f\n', ...
				iC,n_hits(iC,1),scores(iC,1),n_hits(iC,2),scores(iC,2));
end

total_hit	= sum(n_hits,1);
total_score	= total_hit/n_class/n_testing;

fprintf('\n\nFinal result:\n chamfer: %3d / %.4f,   DP: %3d / %.4f\n', ...
			total_hit(1),total_score(1),total_hit(2),total_score(2));
fprintf('\n\n training = %2d, testing = %2d, n_contsamp=%d, n_dist=%d, n_theta=%d \n', ...
		n_training,n_obj-n_training,n_contsamp,n_dist,n_theta);
fprintf('search_step=%d, num_start=%d, match_thre=%f\n', search_step, num_start,match_thre);

fprintf(fRecord,'\n\nFinal result:\n chamfer: %3d / %.4f,   DP: %3d / %.4f\n', ...
			total_hit(1),total_score(1),total_hit(2),total_score(2));
fprintf(fRecord,'\n\n training = %2d, testing = %2d, n_contsamp=%d, n_dist=%d, n_theta=%d \n', ...
		n_training,n_obj-n_training,n_contsamp,n_dist,n_theta);
fprintf(fRecord,'search_step=%d, num_start=%d, match_thre=%f\n', search_step, num_start,match_thre);
fclose(fRecord);

return;